/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package jfileaid;

import javax.swing.table.AbstractTableModel;
import org.nla.cobol.CobolCopyBook;
import org.nla.cobol.CobolCopyBookLineMapping;
import org.nla.cobol.CobolElement;
import org.nla.cobol.CobolElementValueMapping;
import org.nla.cobol.CobolVariable;
import org.nla.utils.Util;

/**
 *
 * @author nico
 */
public class CobolTableModel extends AbstractTableModel {

    protected CobolCopyBook copyBook;

    CobolCopyBookLineMapping copyLineMapping;

    protected boolean hexView;

    public CobolTableModel( CobolCopyBook copyBook ) {
        this.copyBook = copyBook;
    }

    public void setHexView(boolean hexView) {
        this.hexView = hexView;
    }

    public CobolTableModel( CobolCopyBookLineMapping copyLineMapping  ) {
        this.copyLineMapping = copyLineMapping;
    }

    public int getRowCount() {
        if ( copyLineMapping == null ) {
            System.out.println("getRowCount() => copyBook.getCount()");
            return copyBook.getCount();
        }
        else {
            return copyLineMapping.getMappingSize();
        }
    }

    @Override
    public int getColumnCount() {
        if ( copyLineMapping == null ) {
            return 5;
        }
        else {
            return 6;
        }
    }

    @Override
    public boolean isCellEditable(int row, int column)
    {
        if ( copyLineMapping == null ) {
            return false;
        }
        else {
            if ( column == 5 ){
                return true;
            }
        }
        return false;
    }


    public Object getValueAt(int row, int column) {

        if ( copyLineMapping == null ) {
            //System.out.println("getValueAt("+row+","+column+")");
            CobolElement element = copyBook.getListOfCobolElements().get(row);
            switch( column )
            {
                case 0:
                    return ( Util.buildStringOfNTokens(element.getLevel(), " ") + element.getLevelAsString() + " " + element.getAbsoluteName() );
                case 1:
                    return element.getFinalType();
                case 2:
                    return element.getStartPosition();
                case 3:
                    return element.getEndPosition();
                case 4:
                    return element.getSize();
                default:
                    return "[r" + row + ";c" + column + "]";
            }
        }
        else {
            CobolElementValueMapping cevm = copyLineMapping.getMapping( row );
            CobolElement element = cevm.getElement();
            String elementValue = cevm.getElementValue();
            switch( column )
            {
                case 0:
                    return ( Util.buildStringOfNTokens(element.getLevel(), " ") + element.getLevelAsString() + " " + element.getAbsoluteName() );
                case 1:
                    return element.getFinalType();
                case 2:
                    return element.getStartPosition();
                case 3:
                    return element.getEndPosition();
                case 4:
                    return element.getSize();
                case 5:
                {
                    if ( element instanceof CobolVariable ) {
                        return getFormattedValue(elementValue);
                    }
                    else {
                        return "";
                    }

                }
                default:
                    return "[r" + row + ";c" + column + "]";
            }
        }
    }

    public String getColumnName( int column ) {
        switch ( column )
        {
            case 0 : return "Field";
            case 1 : return "Type";
            case 2 : return "Start";
            case 3 : return "End";
            case 4 : return "Size";
            case 5 : return "Value";
            default : return "[c"+column+"]";
        }
    }

    public String getFormattedValue( String elementValue ) {
        if ( hexView ) {
            byte[] valueBytes = elementValue.getBytes();


            StringBuffer buf = new StringBuffer();
            buf.append("X'");
            for ( byte b : valueBytes ) {
                int byteAsInt = (int)b;
                if ( byteAsInt == 0 ) {
                    buf.append( "00" );
                }
                else {
                    buf.append( byteAsInt );
                }
            }
            buf.append("'");

            return buf.toString();
        }
        else {
            return elementValue;
        }
    }

}
